@@ -15,9 +15,10 @@ using Gtk;
 using Gdk;
 
 using FSpot;
-using FSpot.Png;
 using FSpot.UI.Dialog;
 
+using Hyena;
+using Hyena.Widgets;
 using FSpot.Utils;
 
 using Mono.Unix;
@@ -56,63 +57,23 @@ namespace FSpot {
 			done = false;
 		}
 
-		private static void RotateCoefficients (string original_path, RotateDirection direction)
-		{
-			string temporary_path = original_path + ".tmp";	// FIXME make it unique
-			JpegUtils.Transform (original_path, temporary_path, 
-					     direction == RotateDirection.Clockwise ? JpegUtils.TransformType.Rotate90 
-					     : JpegUtils.TransformType.Rotate270);
-			
-			Utils.Unix.Rename (temporary_path, original_path);
-		}
-
 		private static void RotateOrientation (string original_path, RotateDirection direction)
 		{
-			using (FSpot.ImageFile img = FSpot.ImageFile.Create (original_path)) {
-				if (img is JpegFile) {
-					FSpot.JpegFile jimg = img as FSpot.JpegFile;
-					PixbufOrientation orientation = direction == RotateDirection.Clockwise
-						? FSpot.Utils.PixbufUtils.Rotate90 (img.Orientation)
-						: FSpot.Utils.PixbufUtils.Rotate270 (img.Orientation);
-				
-					jimg.SetOrientation (orientation);
-					jimg.SaveMetaData (original_path);
-				} else if (img is PngFile) {
-					PngFile png = img as PngFile;
-					bool supported = false;
+            try {
+                using (var metadata = Metadata.Parse (new SafeUri (original_path))) {
+                    var tag = metadata.ImageTag;
+                    var orientation = direction == RotateDirection.Clockwise
+                        ? FSpot.Utils.PixbufUtils.Rotate90 (tag.Orientation)
+                        : FSpot.Utils.PixbufUtils.Rotate270 (tag.Orientation);
 
-					//FIXME there isn't much png specific here except the check
-					//the pixbuf is an accurate representation of the real file
-					//by checking the depth.  The check should be abstracted and
-					//this code made generic.
-					foreach (PngFile.Chunk c in png.Chunks) {
-						PngFile.IhdrChunk ihdr = c as PngFile.IhdrChunk;
-					
-						if (ihdr != null && ihdr.Depth == 8)
-							supported = true;
-					}
-
-					if (!supported) {
-						throw new RotateException (Catalog.GetString ("Unable to rotate this type of photo"), original_path);
-					}
-
-					string backup = ImageFile.TempPath (original_path);
-					using (Stream stream = File.Open (backup, FileMode.Truncate, FileAccess.Write)) {
-						using (Pixbuf pixbuf = img.Load ()) {
-							PixbufOrientation fake = (direction == RotateDirection.Clockwise) ? PixbufOrientation.RightTop : PixbufOrientation.LeftBottom;
-							using (Pixbuf rotated = FSpot.Utils.PixbufUtils.TransformOrientation (pixbuf, fake)) {
-								img.Save (rotated, stream);
-							}
-						}
-					}
-					File.Copy (backup, original_path, true);
-					File.Delete (backup);
-				} else {
-					throw new RotateException (Catalog.GetString ("Unable to rotate this type of photo"), original_path);
-				}
-			}
+                    tag.Orientation = orientation;
+                    metadata.Save ();
+                }
+            } catch (Exception e) {
+                throw new RotateException (Catalog.GetString ("Unable to rotate this type of photo"), original_path);
+            }
 		}
-		       
+
 		private void Rotate (string original_path, RotateDirection dir)
 		{
 			RotateOrientation (original_path, dir);
@@ -124,7 +85,7 @@ namespace FSpot {
 			if (done)
 				return false;
 
- 			original_path = item.DefaultVersionUri.LocalPath;
+ 			original_path = item.DefaultVersion.Uri.LocalPath;
  			done = true;
 
 			if ((File.GetAttributes(original_path) & FileAttributes.ReadOnly) == FileAttributes.ReadOnly) {
@@ -133,10 +94,6 @@ namespace FSpot {
 
 			Rotate (original_path, direction);
 
-			Gdk.Pixbuf thumb = FSpot.ThumbnailGenerator.Create (UriUtils.PathToFileUri (original_path));
-			if (thumb != null)
-				thumb.Dispose ();
-		
 			return !done;
 		}
 	}
@@ -219,11 +176,11 @@ public class RotateCommand {
 			} catch (GLib.GException) {
 				readonly_count++;
 			} catch (DirectoryNotFoundException e) {
-				RunGenericError (e, op.Items [op.Index].DefaultVersionUri.LocalPath, Catalog.GetString ("Directory not found"));
+				RunGenericError (e, op.Items [op.Index].DefaultVersion.Uri.LocalPath, Catalog.GetString ("Directory not found"));
 			} catch (FileNotFoundException e) {
-				RunGenericError (e, op.Items [op.Index].DefaultVersionUri.LocalPath, Catalog.GetString ("File not found"));
+				RunGenericError (e, op.Items [op.Index].DefaultVersion.Uri.LocalPath, Catalog.GetString ("File not found"));
 			} catch (Exception e) {
-				RunGenericError (e, op.Items [op.Index].DefaultVersionUri.LocalPath);
+				RunGenericError (e, op.Items [op.Index].DefaultVersion.Uri.LocalPath);
 			}
 			index ++;
 		}
